//
// hello.mag.45.c
//
// Hall effect magnetic sensing hello-world
//    9600 baud FTDI interface
//
// Button press on board print out x in serial termianl. x key on keyboard blinks on board LED. 
//
// Ander Haldin modified Neil Gershenfeld code 03/5/15
// (c) Massachusetts Institute of Technology 2013
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input

#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTA
#define serial_direction DDRA

#define serial_pins PINA

#define serial_pin_out (1 << PA1) 
#define serial_pin_in (1 << PA0)

#define BIT(x) (0x01 << (x))// a bit size operation defining  called BIT.


#define node_id '1'

ISR(PCINT0_vect)
{

put_char(&serial_port, serial_pin_out, 'x');
   /*
 if ((PINA & (1<<PINA7)) == 0)
 {
  PORTB |= (1<<PB2); //turn on LED
 } else {
  PORTB &= ~(1<<PB2); //Turn of LED
 }
 */
} //closing ISR Interupt Service Routine


void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //

   bit_delay();
   half_bit_delay();
   }


void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

int main(void) {




   static char chr;
 
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);

   DDRB |= (1<<DDB2); // set as output. set bit in register. LED
   DDRA &= ~(1<<DDA7); // set as input. clear bit in register. BUTTON
    
   PORTB &= ~(1<<PB2); // turn off LED
   PORTA |= (1<<PA7); // enable pull up resistor Button
    
    
   GIMSK |= (1<<PCIE0); //Enable pin change interrupt.
   PCMSK0 |= (1<<PCINT7); //pin mask
   sei(); // enable global interrupt


   //
   // main loop
   //
   while (1) {

   get_char(&serial_pins, serial_pin_in, &chr);
   if (chr == 'c') 
   {
      PORTB |= (1 << PB2); //turn on LED
      _delay_ms(100);
      PORTB &= ~(1<<PB2); // turn off LED
      _delay_ms(100);
      PORTB |= (1 << PB2); //turn on LED
      _delay_ms(100);
      PORTB &= ~(1<<PB2); // turn off LED
      _delay_ms(100);
      PORTB |= (1 << PB2); //turn on LED
      _delay_ms(100);
      PORTB &= ~(1<<PB2); // turn off LED
      _delay_ms(100);

   } //end if statements

 }
}
